#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file ip_route_list.py
 @brief This module gets IP routing table information.
 @date $Date$

 @author 株式会社 東日本計算センター

 NewBSD

"""
import sys
import subprocess
import os
import copy
import IPRouteIF
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV
import netifaces

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
handler.setLevel(LOG_LV)
logger.setLevel(LOG_LV)
logger.addHandler(handler)


class IPRouteList(object):

    def __init__(self):
        self._init_variable()

    def _init_variable(self):
        self._column_count = None
        self._command = None
        self._start_str = None
        self._end_str = None
        self._linefeed = None
        self._destination = None
        self._netmask = None
        self._gateway = None
        self._interface = None
        self._flag = None

    def configure(self):
        is_success = False
        if sys.platform == "win32":
            is_success = self._setup_windows()

        elif sys.platform.startswith('linux'):
            is_success = self._setup_linux()

        else:
            logger.critical("ERROR: NOT supported platform")

        logger.info("platform: " + sys.platform)

        return is_success

    def _setup_windows(self):
        try:
            subprocess.check_output("chcp 437", shell=True)
        except subprocess.CalledProcessError:
            logger.critical("setup error")
            return False

        # ### column name ### #
        # 0: Destination      #
        # 1: Netmask          #
        # 2: Gateway          #
        # 3: Interface        #
        # 4: Metric           #
        # ################### #
        self._column_count = 5
        self._command = "route -4 print"
        self._start_str = "Network Destination"
        self._end_str = "====="
        self._linefeed = "\r\n"
        self._destination = 0
        self._netmask = 1
        self._gateway = 2
        self._interface = 3
        self._flag = None

        return True

    def _setup_linux(self):
        os.environ["LANG"] = "C"

        # ### column name ### #
        # 0: Destination      #
        # 1: Gateway          #
        # 2: Genmask          #
        # 3: Flags            #
        # 4: Metric           #
        # 5: Ref              #
        # 6: Use              #
        # 7: Interface        #
        # ################### #
        self._column_count = 8
        self._command = "route -n"
        self._start_str = "Destination"
        self._end_str = None
        self._linefeed = "\n"
        self._destination = 0
        self._netmask = 2
        self._gateway = 1
        self._interface = 7
        self._flag = 3

        return True

    def get_route_list(self):
        class retval(object):
            is_success = False
            data = []

        result = self._exe_route_cmd()

        if not result.is_success:
            return retval

        for data in result.data:
            data_list = self._parse_output_data(data)

            if not data_list:
                logger.error("ERROR: parse failure")
                return retval

            route_data = self._check_validity(data_list)

            if route_data:
                self._platform_check(route_data)
                retval.data.append(route_data)

        retval.is_success = True

        return retval

    def _platform_check(self, route_data):
        if sys.platform.startswith('linux'):
            # interface name resolution
            resol = self._linux_name_resolution(route_data.iface)
            if resol:
                route_data.iface = copy.deepcopy(resol)

    def _exe_route_cmd(self):
        class retval(object):
            is_success = False
            data = []

        try:
            out = subprocess.check_output(self._command, shell=True)
        except subprocess.CalledProcessError:
            logger.error("ERROR: command execution failure")
            return retval

        is_data_start = False
        record = out.split(self._linefeed)
        record = filter(lambda d: len(d) > 0, record)

        for data in record:
            if is_data_start:
                if self._end_str:
                    if data.startswith(self._end_str):
                        break

                retval.data.append(data)
            else:
                if data.startswith(self._start_str):
                    is_data_start = True

        if not is_data_start:
            logger.error("ERROR: unexpected route data")
            return retval

        retval.is_success = True

        return retval

    def _parse_output_data(self, data):
        data_list = data.split("\x20")
        data_list = filter(lambda d: len(d) > 0, data_list)

        if len(data_list) != self._column_count:
            del data_list[:]

        return data_list

    def _check_validity(self, data_list):
        route_data = None

        is_valid = False
        if self._flag is None:
            if data_list[self._gateway].find("On-link") == -1:
                is_valid = True
        else:
            if data_list[self._flag].find("G") >= 0:
                is_valid = True

        if is_valid:
            route_data = IPRouteIF.Route(data_list[self._destination],
                                         data_list[self._netmask],
                                         data_list[self._gateway],
                                         data_list[self._interface])

        return route_data

    def _linux_name_resolution(self, val):
        retval = []

        retval = netifaces.ifaddresses(val)[netifaces.AF_INET][0]['addr']

        return retval


def debug_ip_route_list():
    m = IPRouteList()
    is_success = m.configure()
    if is_success:
        result = m.get_route_list()

        if result.is_success:
            for data in result.data:
                logger.info(data)

if __name__ == "__main__":
    debug_ip_route_list()
